{#
    cloud/globaltoc.html
    ~~~~~~~~~~~~~~~~~~~
    Wraps <basic/globaltoc.html> with an identifiable div,
    so the page highlight code has a reliable selector to lock onto.
#}
<div class="sphinx-toc sphinxglobaltoc">
  {%- include "basic/globaltoc.html" -%}
</div>
